@@ -91,15 +91,15 @@ def print_logo():
 @click.option('--reset-openai-key', is_flag=True, help='Reset the OpenAI API key')
 @click.pass_context
 def main(ctx, reset_openai_key: bool):
-    """Co-DataScientist: AI-Powered Code Evolution for ML & Data Science
+    """Tropiflo: AI-Powered Code Evolution for ML & Data Science
     
     Automatically evolves your Python code to maximize KPIs through intelligent
     hypothesis-driven optimization. Works with both single files and multi-file projects.
     
     Getting Started:
-        1. co-datascientist set-token --token YOUR_TOKEN
-        2. co-datascientist run --script-path your_project/
-        3. co-datascientist deploy co_datascientist_checkpoints/best_X/
+        1. tropiflo set-token --token YOUR_TOKEN
+        2. tropiflo run --config config.yaml
+        3. tropiflo deploy co_datascientist_checkpoints/best_X/
     
     Core Commands:
         run         - Evolve your code to maximize KPIs
@@ -111,10 +111,10 @@ def main(ctx, reset_openai_key: bool):
         diff-pdf    - Generate beautiful code comparison PDFs
     
     Setup:
-        set-token   - Configure your API key
+        set-token   - Configure your API key (REQUIRED before running)
         openai-key  - Add OpenAI key for unlimited usage
     
-    Use 'co-datascientist COMMAND --help' for detailed info on any command.
+    Use 'tropiflo COMMAND --help' for detailed info on any command.
     """
     # Initialize keyring and logging
     ensure_keyring_works()
@@ -221,6 +221,11 @@ def run(command, working_directory, config_path, use_cached_qa):
         with open(config_path, 'r') as file:
             config_dict = yaml.safe_load(file)
         
+        # Set API key from config if provided (simplest approach!)
+        if 'api_key' in config_dict:
+            from .settings import settings
+            settings.set_api_key_from_config(config_dict['api_key'])
+        
         # Validate and report execution mode
         mode = config_dict.get('mode', '').lower()
         if mode == 'local':
@@ -243,6 +248,10 @@ def run(command, working_directory, config_path, use_cached_qa):
         else:
             print_error("Config must contain 'mode' field or 'gcloud', 'aws', or 'databricks' configuration. Aborting.")
             return
+        
+        # Display engine configuration
+        engine = config_dict.get('engine', 'EVOLVE_HYPOTHESIS').upper()
+        print_info(f"AI Engine: {engine}")
     else:
         print_info("No config provided, using local executor")
         
