@@ -50,7 +50,8 @@ class GCloudExecutor(BaseExecutor):
         self.region = gcloud_config.get("region", "us-central1")
         self.repo = gcloud_config.get("repo", "co-datascientist-repo")
         self.job_name = gcloud_config.get("job_name", "co-datascientist-job")
-        self.cleanup_job = gcloud_config.get("cleanup_job", True)
+        # Default: DON'T cleanup jobs (reuse them for speed), but DO cleanup images (save storage)
+        self.cleanup_job = gcloud_config.get("cleanup_job", False)  # Changed default to False for speed
         self.cleanup_remote_image = gcloud_config.get("cleanup_remote_image", True)
         
         # Data volume support (GCS bucket)
@@ -74,18 +75,34 @@ class GCloudExecutor(BaseExecutor):
         cmd: List[str],
         check: bool = True,
         capture: bool = False,
-        timeout: Optional[int] = None
+        timeout: Optional[int] = None,
+        suppress_output: bool = False
     ) -> subprocess.CompletedProcess:
-        """Run a shell command with logging."""
+        """Run a shell command with logging.
+        
+        Args:
+            suppress_output: If True, redirect stdout and stderr to devnull to silence all output
+        """
         logging.debug(f"Running command: {' '.join(cmd)}")
         try:
-            result = subprocess.run(
-                cmd,
-                check=check,
-                text=True,
-                capture_output=capture,
-                timeout=timeout
-            )
+            if suppress_output:
+                # Completely silence the command
+                result = subprocess.run(
+                    cmd,
+                    check=check,
+                    text=True,
+                    stdout=subprocess.DEVNULL,
+                    stderr=subprocess.DEVNULL,
+                    timeout=timeout
+                )
+            else:
+                result = subprocess.run(
+                    cmd,
+                    check=check,
+                    text=True,
+                    capture_output=capture,
+                    timeout=timeout
+                )
             return result
         except subprocess.CalledProcessError as e:
             logging.error(f"Command failed: {' '.join(cmd)}")
@@ -103,9 +120,11 @@ class GCloudExecutor(BaseExecutor):
                 "gcloud", "artifacts", "repositories", "create", self.repo,
                 "--repository-format=docker",
                 f"--location={self.region}",
-                f"--project={self.project_id}"
+                f"--project={self.project_id}",
+                "--quiet"
             ],
-            check=False
+            check=False,
+            suppress_output=True  # Silence "ALREADY_EXISTS" errors
         )
 
     def _job_exists(self, job_name: str) -> bool:
@@ -116,7 +135,8 @@ class GCloudExecutor(BaseExecutor):
                 f"--region={self.region}",
                 f"--project={self.project_id}"
             ],
-            check=False
+            check=False,
+            suppress_output=True  # Suppress "Cannot find job" errors
         )
         return result.returncode == 0
 
@@ -137,30 +157,16 @@ class GCloudExecutor(BaseExecutor):
         return name or None
 
     def _collect_stdout(self, job_name: str, execution: str) -> str:
-        """Fetch stdout logs for a given execution."""
+        """Fetch stdout logs for a given execution with retry logic."""
         logging.info(f"Collecting logs for execution: {execution}")
         
-        # Try plain text (textPayload) first
-        result = self._run_command(
-            [
-                "gcloud", "logging", "read",
-                f'resource.type="cloud_run_job" '
-                f'AND resource.labels.job_name="{job_name}" '
-                f'AND labels."run.googleapis.com/execution_name"="{execution}"',
-                f"--project={self.project_id}",
-                "--limit=100000",
-                "--format=value(textPayload)",
-            ],
-            check=False,
-            capture=True,
-            timeout=300  # 5 minutes for log collection
-        )
-        
-        payload = result.stdout or ""
-        
-        if not payload.strip():
-            # Fallback: JSON (covers structured logs)
-            logging.info("No textPayload found, trying JSON format")
+        # Retry up to 3 times with 10 second delays (logs can be slow to propagate)
+        for attempt in range(3):
+            if attempt > 0:
+                logging.info(f"Retry {attempt}/2: Waiting for logs to appear...")
+                time.sleep(10)
+            
+            # Try plain text (textPayload) first
             result = self._run_command(
                 [
                     "gcloud", "logging", "read",
@@ -169,36 +175,67 @@ class GCloudExecutor(BaseExecutor):
                     f'AND labels."run.googleapis.com/execution_name"="{execution}"',
                     f"--project={self.project_id}",
                     "--limit=100000",
-                    "--format=json",
+                    "--format=value(textPayload)",
                 ],
                 check=False,
                 capture=True,
-                timeout=300
+                timeout=300  # 5 minutes for log collection
             )
+            
             payload = result.stdout or ""
+            
+            if not payload.strip():
+                # Fallback: JSON (covers structured logs)
+                logging.info("No textPayload found, trying JSON format")
+                result = self._run_command(
+                    [
+                        "gcloud", "logging", "read",
+                        f'resource.type="cloud_run_job" '
+                        f'AND resource.labels.job_name="{job_name}" '
+                        f'AND labels."run.googleapis.com/execution_name"="{execution}"',
+                        f"--project={self.project_id}",
+                        "--limit=100000",
+                        "--format=json",
+                    ],
+                    check=False,
+                    capture=True,
+                    timeout=300
+                )
+                payload = result.stdout or ""
+            
+            # If we got logs, return them
+            if payload.strip():
+                logging.info(f"Successfully collected logs ({len(payload)} bytes)")
+                return payload
         
+        # After all retries, return whatever we have (might be empty)
+        logging.warning(f"Failed to collect logs after 3 attempts")
         return payload
 
     def _push_image(self, local_tag: str, remote_tag: str) -> None:
         """Tag and push image to Artifact Registry."""
         logging.info(f"Pushing image from {local_tag} to {remote_tag}")
+        print(f"   Pushing Docker image to Cloud...")
         
-        # Configure docker auth
+        # Configure docker auth (suppress credential warnings)
         self._run_command(
             ["gcloud", "auth", "configure-docker", f"{self.region}-docker.pkg.dev", "--quiet"],
-            timeout=60
+            timeout=60,
+            suppress_output=True
         )
         
-        # Tag the image
+        # Tag the image (silent)
         self._run_command(
             ["docker", "tag", local_tag, remote_tag],
-            timeout=60
+            timeout=60,
+            suppress_output=True
         )
         
-        # Push the image
+        # Push the image (silent - no layer spam)
         self._run_command(
             ["docker", "push", remote_tag],
-            timeout=1800  # 30 minutes for push
+            timeout=1800,  # 30 minutes for push
+            suppress_output=True
         )
         logging.info(f"Successfully pushed image to {remote_tag}")
 
@@ -213,7 +250,8 @@ class GCloudExecutor(BaseExecutor):
             "gcloud", "run", "jobs", op, job_name,
             "--image", image,
             f"--region={self.region}",
-            f"--project={self.project_id}"
+            f"--project={self.project_id}",
+            "--quiet"
         ]
         
         # Add GCS volume mount if configured
@@ -235,7 +273,7 @@ class GCloudExecutor(BaseExecutor):
             
             logging.info(f"Volume mount: gs://{self.gcs_bucket} -> {mount_path}")
         
-        self._run_command(cmd, timeout=300)  # 5 minutes
+        self._run_command(cmd, timeout=300, suppress_output=True)  # 5 minutes, silent
         logging.info(f"Successfully {op}d Cloud Run Job: {job_name}")
 
     def _execute_job(self, job_name: str, timeout_seconds: int) -> int:
@@ -246,15 +284,18 @@ class GCloudExecutor(BaseExecutor):
         """
         logging.info(f"Executing Cloud Run Job: {job_name}")
         
+        print(f"   Running job on Cloud Run...")
         result = self._run_command(
             [
                 "gcloud", "run", "jobs", "execute", job_name,
                 f"--region={self.region}",
                 f"--project={self.project_id}",
-                "--wait"
+                "--wait",
+                "--quiet"
             ],
             check=False,
-            timeout=timeout_seconds + 300  # Add 5 minutes buffer for overhead
+            timeout=timeout_seconds + 300,  # Add 5 minutes buffer for overhead
+            suppress_output=True  # Suppress all execution spinners
         )
         
         return result.returncode
@@ -270,7 +311,8 @@ class GCloudExecutor(BaseExecutor):
                 "--quiet"
             ],
             check=False,
-            timeout=120
+            timeout=120,
+            suppress_output=True  # Silent cleanup
         )
 
     def _cleanup_image(self, image: str) -> None:
@@ -284,7 +326,8 @@ class GCloudExecutor(BaseExecutor):
                 "--quiet"
             ],
             check=False,
-            timeout=120
+            timeout=120,
+            suppress_output=True  # Silent cleanup
         )
 
     def execute(self, docker_image_tag: str) -> CodeResult:
@@ -299,8 +342,9 @@ class GCloudExecutor(BaseExecutor):
         """
         start_time = time.time()
         
-        # Use consistent job name (will be updated if it exists)
-        job_name = self.job_name
+        # Use UNIQUE job name per execution to avoid conflicts in parallel mode
+        # Append docker_image_tag to ensure uniqueness
+        job_name = f"{self.job_name}-{docker_image_tag}"
         
         # Construct remote image path
         ar_image = f"{self.region}-docker.pkg.dev/{self.project_id}/{self.repo}/{docker_image_tag}"
@@ -313,22 +357,25 @@ class GCloudExecutor(BaseExecutor):
         return_code = -1
         
         try:
-            # Set project
+            # Set project (suppress output)
             self._run_command(
-                ["gcloud", "config", "set", "project", self.project_id],
-                timeout=30
+                ["gcloud", "config", "set", "project", self.project_id, "--quiet"],
+                timeout=30,
+                suppress_output=True
             )
             
-            # Enable required APIs (no-op if already enabled)
+            # Enable required APIs (no-op if already enabled, suppress output)
             logging.info("Ensuring required GCP APIs are enabled")
             self._run_command(
                 [
                     "gcloud", "services", "enable",
                     "artifactregistry.googleapis.com",
-                    "run.googleapis.com"
+                    "run.googleapis.com",
+                    "--quiet"
                 ],
                 check=False,
-                timeout=120
+                timeout=120,
+                suppress_output=True
             )
             
             # Ensure Artifact Registry repo exists
@@ -343,6 +390,11 @@ class GCloudExecutor(BaseExecutor):
             # Execute the job
             return_code = self._execute_job(job_name, settings.script_execution_timeout)
             
+            # Wait for logs to propagate to Cloud Logging (logs can take 10-30s after job completes)
+            print("   Collecting logs from Cloud Run...")
+            logging.info("Waiting for logs to propagate to Cloud Logging...")
+            time.sleep(15)  # Give logs time to propagate
+            
             # Collect logs
             exec_name = self._latest_execution(job_name)
             if exec_name:
