@@ -21,6 +21,8 @@ from .settings import settings
 from .qa_cache import get_answers, QACache
 from .user_steering import get_steering_handler, wrap_spinner_with_coordination
 from .executors import BaseExecutor
+from .environment_detector import EnvironmentDetector
+from .docker_generator import DockerGenerator
 
 OUTPUT_FOLDER = "co_datascientist_output"
 CHECKPOINTS_FOLDER = "co_datascientist_checkpoints"
@@ -56,10 +58,13 @@ def ignore_dirs(dir, files):
     return ignore_list
 
 
-def move_to_tmp(code_base_directory):
+def copy_to_tmp(code_base_directory):
     """
-    Copy the code_base_directory to a temporary directory, ignoring
-    'current_runs' and 'co_datascientist_checkpoints' directories.
+    SAFELY COPY the code_base_directory to a temporary directory.
+    
+    IMPORTANT: This is a COPY operation - your original workspace is never modified!
+    
+    Ignores 'current_runs' and 'co_datascientist_checkpoints' directories.
     Returns the path to the copied code base directory in the temp location.
     """
     import shutil
@@ -102,13 +107,67 @@ class WorkflowRunner:
         self.steering_handler = get_steering_handler()
         self._steering_bar_started: bool = False
 
-    def prep_workflow(self, working_directory: str):
-        """Prep the workflow by moving the code to temp and getting the run command for the docker and the files to evolve out."""
-        temp_code_base_directory = move_to_tmp(working_directory)
+    def prep_workflow(self, working_directory: str, config: dict):
+        """Prep the workflow by SAFELY COPYING the code to temp (original workspace untouched) and setting up Docker."""
+        temp_code_base_directory = copy_to_tmp(working_directory)
         code_base_directory, dockerfile_path = auto_detect_repository_structure(temp_code_base_directory)
+        
+        # Auto-generate Docker setup if Dockerfile doesn't exist
+        if not dockerfile_path.exists():
+            print("   No Dockerfile found - auto-generating from your environment...")
+            dockerfile_path = self._auto_generate_docker_setup(
+                working_directory=working_directory,
+                temp_directory=temp_code_base_directory,
+                config=config
+            )
+            print("   Docker setup generated automatically")
 
         return code_base_directory, dockerfile_path
     
+    def _auto_generate_docker_setup(self, working_directory: str, temp_directory: Path, config: dict) -> Path:
+        """
+        Auto-generate Dockerfile and requirements.txt from user's environment.
+        User never needs to create these files manually!
+        
+        Args:
+            working_directory: Original project directory
+            temp_directory: Temp directory where Docker files will be written
+            config: User config containing entry_command
+        
+        Returns:
+            Path to generated Dockerfile
+        """
+        # Get entry command from config
+        entry_command = config.get('entry_command')
+        if not entry_command:
+            raise ValueError(
+                "ERROR: No 'entry_command' specified in config.yaml\n"
+                "   Please add: entry_command: 'python your_script.py'\n"
+                "   This is the command you normally use to run your code."
+            )
+        
+        # Detect environment from user's current setup
+        detector = EnvironmentDetector(working_directory)
+        env_snapshot = detector.get_environment_snapshot()
+        
+        # Generate Docker files
+        generator = DockerGenerator()
+        dockerfile_content, requirements_content = generator.generate_dockerfile(
+            python_version=env_snapshot['python_version'],
+            requirements=env_snapshot['requirements'],
+            entry_command=entry_command,
+            working_directory=Path(working_directory),
+        )
+        
+        # Write to temp directory
+        dockerfile_path = generator.write_docker_files(
+            temp_directory=temp_directory,
+            dockerfile_content=dockerfile_content,
+            requirements_content=requirements_content,
+        )
+        
+        return dockerfile_path
+    
 
     def get_evolvable_files(self, working_dir_path: str) -> dict[str, str]:
         """
@@ -184,7 +243,7 @@ class WorkflowRunner:
 
         # Generate a unique image tag
         image_tag = f"co-datascientist-{uuid.uuid4().hex[:8]}"
-        print("Dockerfile path: ", dockerfile_path)
+        logging.debug(f"Dockerfile path: {dockerfile_path}")
         # Build the docker image
         # The build context should be the directory containing the Dockerfile, not the Dockerfile itself
         build_context = dockerfile_path.parent
@@ -201,7 +260,7 @@ class WorkflowRunner:
                 capture_output=True,
                 text=True
             )
-            print(f"Docker image built successfully: {image_tag}")
+            logging.info(f"Docker image built: {image_tag}")
         except subprocess.CalledProcessError as e:
             print(f"Error building Docker image:\n{e.stderr}")
             raise
@@ -237,7 +296,7 @@ class WorkflowRunner:
         # compile the users docker image in temp now and then run it from temp early so we know if there are crashes before the preflight....
         self.stitch_evolvable_files(evolvable_files, temp_code_base_directory)
         docker_image_tag = self.compile_docker_image(docker_file_directory)   
-        print("Executing the baseline docker image")
+        logging.info("Executing code in Docker container")
         try:
             # Run the blocking executor.execute() in a thread pool so it doesn't block the event loop
             code_result = await asyncio.to_thread(executor.execute, docker_image_tag)
@@ -275,7 +334,7 @@ class WorkflowRunner:
 
         evolvable_files = self.get_evolvable_files(working_directory) # get a dict of python files from this dir ... 
         assert len(evolvable_files) > 0, "No blocks found in any Python file in the directory."
-        temp_code_base_directory, docker_file_directory = self.prep_workflow(working_directory) # move the users workspace to temp and find the docker file.
+        temp_code_base_directory, docker_file_directory = self.prep_workflow(working_directory, config) # COPY user's workspace to temp (original untouched) and find/generate Dockerfile
         
         executor = ExecutorFactory.create_executor(python_path="python", config=config)
         print("Running your baseline to start")
@@ -293,7 +352,13 @@ class WorkflowRunner:
                 print(f"   Standard output:\n{result.stdout}")
             assert False, "Baseline docker image failed to execute"
         
-        print(f"Baseline result: {result}")
+        # Show clean baseline result
+        if result.kpi is not None:
+            print(f"Baseline KPI: {result.kpi} (runtime: {result.runtime_ms:.0f}ms)")
+        else:
+            print(f"Baseline completed (runtime: {result.runtime_ms:.0f}ms) - No KPI extracted")
+        print("--------------------------------")
+        logging.debug(f"Full baseline result: {result}")
 
         # TODO: get the types of all these parts of the model of what needs to be sent through! 
 
@@ -311,8 +376,10 @@ class WorkflowRunner:
         self.start_timestamp = time.time()
         # system_info = get_system_info(python_path) #TODO: we may be able to get this from the docker ime... for now just go with None
         system_info = SystemInfo(python_libraries=["None"],python_version="3.13",os=sys.platform)
+        # Get engine from config (default to EVOLVE_HYPOTHESIS)
+        engine = config.get('engine', 'EVOLVE_HYPOTHESIS').upper()
         # Start preflight: send full dict of evolvable files
-        preflight = await co_datascientist_api.start_preflight(evolvable_files, system_info) 
+        preflight = await co_datascientist_api.start_preflight(evolvable_files, system_info, engine) 
         self.workflow = preflight.workflow
         # Stop spinner to allow clean input UX
         if spinner:
@@ -327,6 +394,26 @@ class WorkflowRunner:
         # Complete preflight: engine summarizes and starts baseline
         initial_response = await co_datascientist_api.complete_preflight(self.workflow.workflow_id, answers)
         self.workflow = initial_response.workflow
+        
+        # Show user what the AI understands about their problem
+        # Use engine's summary if available, otherwise show Q&A context
+        if self.workflow.user_context_summary:
+            print(f"\n{self.workflow.user_context_summary}\n")
+        elif observation:
+            # Show the engine's observation about the code
+            print(f"\n{observation}")
+            if questions and answers:
+                print("\nYour optimization preferences:")
+                for q, a in zip(questions, answers):
+                    print(f"  • {a}")
+            print()  # blank line
+        elif questions and answers:
+            # Fallback: just show answers if no observation
+            print("\nOptimization preferences:")
+            for a in answers:
+                if a.strip():  # Only non-empty answers
+                    print(f"  • {a}")
+            print()  # blank line
 
         # Exception: Failed to process batch results: Code version baseline not found in batch! DEBUG.
         # TODO: ok so we really need to understnad the exact thing that needs to be posted back... this is the sticking piint now.
@@ -351,14 +438,34 @@ class WorkflowRunner:
                 spinner.text = f"Running {batch_size} programs in parallel..."
                 spinner.start()
             
-            ###WHY DONT WE GET HERE!!!!???  i does some v weerid thing in the finished runnign batch first one!!!!!
-            print(f"Batch size: {batch_size}")
-            # Check for both None and empty list - backend returns [] when generation fails
+            # Let user know we're generating hypotheses
+            print(f"\nGenerating {batch_size} new hypotheses...")
+            
+            # Request batch from backend and measure generation time
+            batch_start = time.time()
             batch_to_run = await co_datascientist_api.get_batch_to_run(self.workflow.workflow_id, batch_size=batch_size)
+            batch_elapsed = time.time() - batch_start
+            
             self.workflow = batch_to_run.workflow
             # We have a batch to run!
             code_versions = batch_to_run.batch_to_run
             batch_id = batch_to_run.batch_id
+            
+            # Show user how quickly hypotheses were generated
+            num_hypotheses = len(code_versions)
+            if num_hypotheses > 0:
+                time_per_hypothesis = batch_elapsed / num_hypotheses
+                print(f"\n{num_hypotheses} hypotheses generated in {batch_elapsed:.1f}s ({time_per_hypothesis:.2f}s per idea)")
+                print(f"Batch size: {batch_size}\n")
+                
+                # Print all hypotheses before testing
+                print("Generated Hypotheses:")
+                print("-" * 70)
+                for i, cv in enumerate(code_versions, 1):
+                    hypothesis_text = cv.hypothesis or "No hypothesis description"
+                    print(f"{i}. {hypothesis_text}")
+                print("-" * 70)
+                print(f"\nTesting {num_hypotheses} hypotheses...\n")
 
             # Run the executor in parallel for the batch using asyncio.gather
             import asyncio
@@ -387,8 +494,9 @@ class WorkflowRunner:
                     self.stich_compile_execute(evolvable_files, temp_code_dir_for_version, docker_file_path_for_version, executor)
                 )
             code_results = await asyncio.gather(*tasks)
+            # Log results for debugging (user-facing output comes later with hypothesis)
             for code_result in code_results:
-                print(f"Code result: {code_result}")
+                logging.debug(f"Code execution result: {code_result}")
             # Cleanup temp dirs after execution
             for temp_dir in temp_dirs:
                 try:
@@ -595,19 +703,24 @@ class WorkflowRunner:
                 if kpi_value is not None and result.return_code == 0:
                     baseline_kpi = self._get_baseline_kpi()
                     hypothesis_outcome = baseline_kpi < kpi_value if baseline_kpi is not None else None
-                    print()
+                    print("\n" + "-"*70)
                     print(f"Hypothesis: {cv.hypothesis or 'Unknown hypothesis'}")
-                    print(f" - Result: {hypothesis_outcome}, KPI: {kpi_value}")
-                    print("--------------------------------")
+                    if hypothesis_outcome is True:
+                        print(f"   [IMPROVED] KPI: {kpi_value} (baseline: {baseline_kpi})")
+                    elif hypothesis_outcome is False:
+                        print(f"   [WORSE] KPI: {kpi_value} (baseline: {baseline_kpi})")
+                    else:
+                        print(f"   [RESULT] KPI: {kpi_value}")
+                    print("-"*70)
                 else:
                     # Handle failed executions like parallel mode
-                    print()
+                    print("\n" + "-"*70)
                     print(f"Hypothesis: {cv.hypothesis or 'Unknown hypothesis'}")
                     if getattr(cv, 'hypothesis_outcome', None) == "failed":
-                        print(" - Failed after all retries - moving on")
+                        print("   [FAILED] After all retries - moving on")
                     else:
-                        print(" - Debugging and queuing for retry...")
-                    print("--------------------------------")
+                        print("   [DEBUG] Queuing for retry...")
+                    print("-"*70)
         else:
             # Parallel mode: existing parallel display logic
             baseline_kpi = self._get_baseline_kpi()
@@ -625,26 +738,30 @@ class WorkflowRunner:
                 else:
                     failed_results.append((cv, res))
             
+            # Display results with thin separator
+            print("\n" + "-"*70)
+            
             # Display successful results
             for cv, kpi_value, hypothesis_outcome in successful_results:
                 print()
                 print(f"Hypothesis: {cv.hypothesis or 'Unknown hypothesis'}")
-                if hypothesis_outcome is not None:
-                    print(f" - Result: {hypothesis_outcome}, KPI: {kpi_value}")
+                if hypothesis_outcome is True:
+                    print(f"   [IMPROVED] KPI: {kpi_value} (baseline: {baseline_kpi})")
+                elif hypothesis_outcome is False:
+                    print(f"   [WORSE] KPI: {kpi_value} (baseline: {baseline_kpi})")
                 else:
-                    print(f" - Result: KPI = {kpi_value}")
+                    print(f"   [RESULT] KPI: {kpi_value}")
             
             # Display failed results - show debugging status
             for cv, res in failed_results:
                 print()
                 print(f"Hypothesis: {cv.hypothesis or 'Unknown hypothesis'}")
                 if getattr(cv, 'hypothesis_outcome', None) == "failed":
-                    print(" - Failed after all retries - moving on")
+                    print("   [FAILED] After all retries - moving on")
                 else:
-                    print(" - Debugging and queuing for retry...")
+                    print("   [DEBUG] Queuing for retry...")
             
-            if code_versions:
-                print("--------------------------------")
+            print("-"*70)
 
     def _get_baseline_kpi(self):
         """Get baseline KPI for comparison."""
